from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Any, Callable, Tuple, Union

if TYPE_CHECKING:
    from collections.abc import Mapping

    from capsula._decorator import CapsuleParams
    from capsula.utils import ExceptionInfo

    from ._backport import Self, TypeAlias


_ContextKey: TypeAlias = Union[str, Tuple[str, ...]]


class Capsule:
    def __init__(
        self,
        data: Mapping[_ContextKey, Any],
        fails: Mapping[_ContextKey, ExceptionInfo],
    ) -> None:
        self.data = dict(data)
        self.fails = dict(fails)


class CapsuleItem(ABC):
    @abstractmethod
    def encapsulate(self) -> Any:
        raise NotImplementedError

    def default_key(self) -> str | tuple[str, ...]:
        msg = f"{self.__class__.__name__}.default_key() is not implemented"
        raise NotImplementedError(msg)

    @classmethod
    def default(cls, *args: Any, **kwargs: Any) -> Callable[[CapsuleParams], Self]:
        def callback(params: CapsuleParams) -> Self:  # type: ignore[type-var,misc] # noqa: ARG001
            return cls(*args, **kwargs)

        return callback
